@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2004 - 2005 Tomasz Kojm <tkojm@clamav.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -180,7 +180,7 @@ int cli_bm_scanbuff(const char *buffer, unsigned int length, const char **virnam
 		    if(p->target || p->offset) {
 			off = offset + i - BM_MIN_LENGTH + BM_BLOCK_SIZE;
 
-			if(fd == -1 || !cli_validatesig(p->target, ftype, p->offset, off, fd, p->virname)) {
+			if((fd == -1 && !ftype) || !cli_validatesig(p->target, ftype, p->offset, off, fd, p->virname)) {
 			    p = p->next;
 			    continue;
 			}
