@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002 - 2004 Tomasz Kojm <tkojm@clamav.net>
+ *  Copyright (C) 2002 - 2005 Tomasz Kojm <tkojm@clamav.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -52,9 +52,9 @@
 pthread_mutex_t logg_mutex = PTHREAD_MUTEX_INITIALIZER;
 #endif
 
-FILE *logg_fd = NULL;
+FILE *logg_fs = NULL;
 
-short int logg_verbose = 0, logg_lock = 0, logg_time = 0;
+short int logg_verbose = 0, logg_lock = 1, logg_time = 0;
 int logg_size = 0;
 const char *logg_file = NULL;
 #if defined(USE_SYSLOG) && !defined(C_AIX)
@@ -71,10 +71,16 @@ int mdprintf(int desc, const char *str, ...)
 	int bytes;
 
     va_start(args, str);
-    bytes = vsnprintf(buff, 512, str, args);
+    bytes = vsnprintf(buff, sizeof(buff), str, args);
     va_end(args);
-    write(desc, buff, bytes);
-    return bytes;
+
+    if(bytes == -1)
+	return bytes;
+
+    if(bytes >= sizeof(buff))
+	bytes = sizeof(buff) - 1;
+
+    return send(desc, buff, bytes, 0);
 }
 
 void logg_close(void) {
@@ -82,9 +88,9 @@ void logg_close(void) {
 #ifdef CL_THREAD_SAFE
     pthread_mutex_lock(&logg_mutex);
 #endif
-    if (logg_fd) {
-	fclose(logg_fd);
-	logg_fd = NULL;
+    if (logg_fs) {
+	fclose(logg_fs);
+	logg_fs = NULL;
     }
 #ifdef CL_THREAD_SAFE
     pthread_mutex_unlock(&logg_mutex);
@@ -99,7 +105,7 @@ void logg_close(void) {
 
 int logg(const char *str, ...)
 {
-	va_list args;
+	va_list args, argscpy;
 	struct flock fl;
 	char *pt, *timestr, vbuff[1025];
 	time_t currtime;
@@ -108,26 +114,28 @@ int logg(const char *str, ...)
 
 
     va_start(args, str);
+    /* va_copy is less portable so we just use va_start once more */
+    va_start(argscpy, str);
 
     if(logg_file) {
 #ifdef CL_THREAD_SAFE
 	pthread_mutex_lock(&logg_mutex);
 #endif
-	if(!logg_fd) {
+	if(!logg_fs) {
 	    old_umask = umask(0037);
-	    if((logg_fd = fopen(logg_file, "a")) == NULL) {
+	    if((logg_fs = fopen(logg_file, "a")) == NULL) {
 		umask(old_umask);
 #ifdef CL_THREAD_SAFE
 		pthread_mutex_unlock(&logg_mutex);
 #endif
-		printf("ERROR: Can't open %s in append mode.\n", logg_file);
+		printf("ERROR: Can't open %s in append mode (check permissions!).\n", logg_file);
 		return -1;
 	    } else umask(old_umask);
 
 	    if(logg_lock) {
 		memset(&fl, 0, sizeof(fl));
 		fl.l_type = F_WRLCK;
-		if(fcntl(fileno(logg_fd), F_SETLK, &fl) == -1) {
+		if(fcntl(fileno(logg_fs), F_SETLK, &fl) == -1) {
 #ifdef CL_THREAD_SAFE
 		    pthread_mutex_unlock(&logg_mutex);
 #endif
@@ -144,7 +152,7 @@ int logg(const char *str, ...)
 	    pt = ctime(&currtime);
 	    timestr = mcalloc(strlen(pt), sizeof(char));
 	    strncpy(timestr, pt, strlen(pt) - 1);
-	    fprintf(logg_fd, "%s -> ", timestr);
+	    fprintf(logg_fs, "%s -> ", timestr);
 	    free(timestr);
 	}
 
@@ -152,10 +160,10 @@ int logg(const char *str, ...)
 	    if(stat(logg_file, &sb) != -1) {
 		if(sb.st_size > logg_size) {
 		    logg_file = NULL;
-		    fprintf(logg_fd, "Log size = %d, maximal = %d\n", (int) sb.st_size, logg_size);
-		    fprintf(logg_fd, "LOGGING DISABLED (Maximal log file size exceeded).\n");
-		    fclose(logg_fd);
-		    logg_fd = NULL;
+		    fprintf(logg_fs, "Log size = %d, maximal = %d\n", (int) sb.st_size, logg_size);
+		    fprintf(logg_fs, "LOGGING DISABLED (Maximal log file size exceeded).\n");
+		    fclose(logg_fs);
+		    logg_fs = NULL;
 #ifdef CL_THREAD_SAFE
 		    pthread_mutex_unlock(&logg_mutex);
 #endif
@@ -166,18 +174,18 @@ int logg(const char *str, ...)
 
 
 	if(*str == '!') {
-	    fprintf(logg_fd, "ERROR: ");
-	    vfprintf(logg_fd, str + 1, args);
+	    fprintf(logg_fs, "ERROR: ");
+	    vfprintf(logg_fs, str + 1, args);
 	} else if(*str == '^') {
-	    fprintf(logg_fd, "WARNING: ");
-	    vfprintf(logg_fd, str + 1, args);
+	    fprintf(logg_fs, "WARNING: ");
+	    vfprintf(logg_fs, str + 1, args);
 	} else if(*str == '*') {
 	    if(logg_verbose)
-		vfprintf(logg_fd, str + 1, args);
-	} else vfprintf(logg_fd, str, args);
+		vfprintf(logg_fs, str + 1, args);
+	} else vfprintf(logg_fs, str, args);
 
 
-	fflush(logg_fd);
+	fflush(logg_fs);
 
 #ifdef CL_THREAD_SAFE
 	pthread_mutex_unlock(&logg_mutex);
@@ -186,39 +194,30 @@ int logg(const char *str, ...)
 
 #if defined(USE_SYSLOG) && !defined(C_AIX)
     if(logg_syslog) {
-
-	/* due to a problem with superfluous control characters (which
-	 * vsnprintf() handles correctly) in (v)syslog we have to remove
-	 * them in a final string
-	 *
-	 * FIXME: substitute %% instead of _
-	 */
-	vsnprintf(vbuff, 1024, str, args);
+	vsnprintf(vbuff, 1024, str, argscpy);
 	vbuff[1024] = 0;
 
-	while((pt = strchr(vbuff, '%')))
-	    *pt = '_';
-
 	if(vbuff[0] == '!') {
-	    syslog(LOG_ERR, vbuff + 1);
+	    syslog(LOG_ERR, "%s", vbuff + 1);
 	} else if(vbuff[0] == '^') {
-	    syslog(LOG_WARNING, vbuff + 1);
+	    syslog(LOG_WARNING, "%s", vbuff + 1);
 	} else if(vbuff[0] == '*') {
 	    if(logg_verbose) {
-		syslog(LOG_DEBUG, vbuff + 1);
+		syslog(LOG_DEBUG, "%s", vbuff + 1);
 	    }
-	} else syslog(LOG_INFO, vbuff);
+	} else syslog(LOG_INFO, "%s", vbuff);
 
     }
 #endif
 
     va_end(args);
+    va_end(argscpy);
     return 0;
 }
 
 void mprintf(const char *str, ...)
 {
-	va_list args;
+	va_list args, argscpy;
 	FILE *fd;
 	char logbuf[512];
 
@@ -237,10 +236,7 @@ void mprintf(const char *str, ...)
 	return;
     }
 
-    if(mprintf_stdout)
-	fd = stdout;
-    else
-	fd = stderr;
+    fd = stdout;
 
 /* legend:
  * ! - error
@@ -250,30 +246,38 @@ void mprintf(const char *str, ...)
 
 /*
  *             ERROR    WARNING    STANDARD
- * normal       yes       yes        yes
+ * normal      stderr   stderr     stdout
  * 
- * verbose      yes       yes        yes
+ * verbose     stderr   stderr     stdout
  * 
- * quiet        yes       no         no
+ * quiet       stderr     no         no
  */
 
 
     va_start(args, str);
+    /* va_copy is less portable so we just use va_start once more */
+    va_start(argscpy, str);
 
     if(*str == '!') {
+       if(!mprintf_stdout)
+           fd = stderr;
 	fprintf(fd, "ERROR: ");
 	vfprintf(fd, ++str, args);
     } else if(*str == '@') {
+       if(!mprintf_stdout)
+           fd = stderr;
 	fprintf(fd, "ERROR: ");
 	vfprintf(fd, ++str, args);
 #ifdef NO_SNPRINTF
-	vsprintf(logbuf, str, args);
+	vsprintf(logbuf, str, argscpy);
 #else
-	vsnprintf(logbuf, sizeof(logbuf), str, args);
+	vsnprintf(logbuf, sizeof(logbuf), str, argscpy);
 #endif
 	logg("ERROR: %s", logbuf);
     } else if(!mprintf_quiet) {
 	if(*str == '^') {
+           if(!mprintf_stdout)
+               fd = stderr;
 	    fprintf(fd, "WARNING: ");
 	    vfprintf(fd, ++str, args);
 	} else if(*str == '*') {
@@ -283,6 +287,7 @@ void mprintf(const char *str, ...)
     }
 
     va_end(args);
+    va_end(argscpy);
 
     if(fd == stdout)
 	fflush(stdout);
